<div align="center">

<img src="https://assets-global.website-files.com/5e68f0772de982756aa8c1a4/5eee5fb470215e6ecdc34b94_Full_transparent_black_1280x413.svg" height=50>
<br>

# JavaScript SDK

[![pipeline status](https://gitlab.com/notabene/open-source/javascript-sdk/badges/master/pipeline.svg)](https://gitlab.com/notabene/open-source/javascript-sdk/-/commits/master)
[![Latest Release](https://gitlab.com/notabene/open-source/javascript-sdk/-/badges/release.svg)](https://gitlab.com/notabene/open-source/javascript-sdk/-/releases)

This library is the JavaScript SDK for loading the widget on a frontend.

[Documentation](https://devx.notabene.id/docs/widget-v2) •
[Installation](#installation)

</div>

## Installation

There are two options for loading the Notabene SDK:

```bash
<script id="notabene" async src="https://unpkg.com/@notabene/javascript-sdk@1.19.0/dist/es/index.js"></script>
```

Or installing the library:

```bash
yarn add @notabene/javascript-sdk
```

## Usage

Create a new Notabene instance:

```js
const notabene = new Notabene({
  vaspDID: 'did:ethr:0x94c38fd29ef36f5cb2f7bc771f9d5bd9f7d05f27',
  widget: 'https://beta-widget.notabene.id',
  container: '#container',
  authToken: '{CUSTOMER_TOKEN}',
  onValidStateChange: (isValid) => {
    // Use this value to determine if the transaction is ready to be created.
    console.log('is transaction valid', isValid);
  },
  onError: (err) => {
    // If any errors are encountered, they will be passed to this function
    alert(err.message);
  },
});
```

Then render the widget:

```js
// Use this method when you need to collect missing information from the sender about the recipient (normal withdrawal)
notabene.renderWidget('WITHDRAWAL');

// Use this method when you need to collect missing information from the recipient about the sender (for transactions created via notification)
notabene.renderWidget('POST_DEPOSIT');
```

To update the widget as users enter transaction details:

```js
notabene.setTransaction({
  transactionAsset: 'ETH',
  beneficiaryAccountNumber: '0x8d12a197cb00d4747a1fe03395095ce2a5cc6819',
  transactionAmount: '2000000000',
});
```

Once the widget determines the transaction is valid, it will call the `onValidStateChange` callback, to access the transaction details:

```js
const currentTransactionInfo = notabene.tx;
```

###Supported asset formats

- `USDC` the simple asset code passed as a `string`, in case different chain (polygon) -> `USDC-POLY`
- `CAIP19` format
  ```js
  {
    caip19: 'eip155:1/erc20:0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48'; // for USDC
  }
  ```
- `coingeckoId` and `network` format
  ```js
  {
    coingeckoId: "usd-coin",
    network: "ethereum"
  }
  ```

---

### Re-rendering

**WARNING:** this shouldn't be used often, if you need to change the transaction's initial fields call `setTransaction` again. (This should be used only for tearing down the component).

The container of the widget must be in the DOM before calling `destroyWidget` or `renderWidget`.

If you need to close and re-render the widget, call the `destroyWidget` method like so:

```js
// Will remove the widget
notabene.destroyWidget();
// Will re-render the widget
notabene.renderWidget();
```

Calling the `renderWidget` methods without destroying it first will not work.

## Customization

**Functionality**

`transactionTypeAllowed`: `ALL` | `VASP_2_VASP_ONLY` | `SELF_TRANSACTION_ONLY`

For limiting the type of transaction destinations you can pass.

`ALL`: All transaction destinations are allowed. (DEFAULT)
`VASP_2_VASP_ONLY`: Only transaction destinations that are VASPs are allowed.
`SELF_TRANSACTION_ONLY`: Only transaction destinations that the originator owns are allowed.

---

`nonCustodialDeclarationType`: `SIGNATURE` | `DECLARATION`

For deciding which ownership proof type you want to use.

`SIGNATURE`: The ownership proof will be signed by the originator using a wallet. (DEFAULT)
`DECLARATION`: The ownership proof will be declared by the originator using a checkbox.

**Pass the variables in the configuration**

```js
const notabene = new Notabene({
    vaspDID: 'did:ethr:0x94c38fd29ef36f5cb2f7bc771f9d5bd9f7d05f27',
    widget: 'https://beta-widget.notabene.id',
    container: '#container',
    authToken: '{CUSTOMER_TOKEN}'
    allowedTransactionTypes: 'VASP_2_VASP_ONLY',
    nonCustodialDeclarationType: 'DECLARATION',
});
```

---

**Theme**

Pass a `theme` object when creating an instance

Here you can pass configuration which will customize the styles of the widget to meet your brand needs, all values are optional.

```js
{
  primaryColor: '#fff',
  secondaryColor: '#fff',
  primaryFontColor: '#fff',
  secondaryFontColor: '#fff',
  backgroundColor: '#fff',
  fontFamily: Arial,
  logo: {LOGO_URL},
  mode: 'dark', // default: 'light'
}
```

To get a list of supported fonts, visit [https://fonts.google.com/](https://fonts.google.com/)

---

**Dictionary**

Pass a `dictionary` object mapping in text in the widget to your own language, for example:

To replace `Loading...` with `Cargando...`

```js
{
  'Loading...': 'Cargando...',
}
```

###### The property name must be the same as the text in the widget, for it to be replaced

---

**Fields Properties**

Possible fields customization support:

| field name               | `forceDisplay` | `optional` | description                                             |
| ------------------------ | -------------- | ---------- | ------------------------------------------------------- |
| `counterparty`           | --             | ☑️         | Counterparty VASP or Wallet (destination or originator) |
| `dateAndPlaceOfBirth`    | ☑️             | --         | Recipient (or Sender) Date and Place of Birth           |
| `geographicAddress`      | ☑️             | ☑️         | Recipient (or Sender) Address                           |
| `name`                   | ☑️             | --         | Recipient (or Sender) Name                              |
| `nationalIdentification` | ☑️             | --️        | Recipient (or Sender) National Identification           |

**Field Options**

- **forceDisplay** - Force a field that is not required by your jurisdiction to be displayed - defaults to **false**
- **optional** - Bypass the jurisdiction rules validation for that specific field - defaults to **false**

```js
{
  counterparty: {
    optional: true;
  },
  geographicAddress: {
    forceDisplay: true,
    optional: true;
  },
}
```

---

**_Beneficiary Details_ [DEPRECATED]**

You can require specific beneficiary fields that are not required by your jurisdiction.

Possible fields:

- `beneficiaryName` - Recipient Name
- `beneficiaryGeographicAddress` - Recipient Address
- `beneficiaryNationalIdentification` - Recipient National Identification
- `beneficiaryDateAndPlaceOfBirth` - Recipient Date and Place of Birth

```js
beneficiaryDetails: [
  'benficiaryName',
  'beneficiaryGeographicAddress',
  'beneficiaryNationalIdentification',
  'beneficiaryDateAndPlaceOfBirth',
];
```

---

**Fallbacks**

Fallbacks are custom options provided when desired outcome is not achieved

Possible options and actions:

- `WALLET_NOT_SUPPORTED` - Performs one of the following options when wallet is not supported
  - `DECLARATION` - Falls back to the self declaration flow
  - `REJECT` - Rejects the transaction and throws an error
    - The error that will be thrown here will have the following structure
      ```js
        {
          "type": "WALLET_NOT_SUPPORTED",
          "title": "Wallet Not Supported",
          "status": 501,
          "detail": "We don't support the wallet for the provided asset"
        }
      ```

```js
fallbacks: [{flow: "WALLET_NOT_SUPPORTED", action: "DECLARATION"}]
```
---

Finally you pass the configuration to a Notabene instance:

```js
const notabene = new Notabene({
    widget: 'https://beta-widget.notabene.id',
    container: '#container',
    authToken: '{CUSTOMER_TOKEN}'
    theme: {THEME},
    dictionary: {DICTIONARY},
    beneficiaryDetails: {BENEFICIARY_DETAILS},
    fallbacks: [{flow: "WALLET_NOT_SUPPORTED", action: "DECLARATION"}]
]
});
```

## [License](LICENSE.md)

BSD 3-Clause © Notabene Inc.
